%
%   TANKCS1.M  This file simulates the single holding tank system described in 
%              Section 6B of the Lecture Notes (see notes for detailed explanation)
%
%   Simulation of Simple Fluid Flow Systems (Case Study B - Case 1)
%
%   File prepared by J. R. White, UMass-Lowell (last update: Feb. 2020)
%

      clear all,   close all,   nfig = 0;
      
%   data for simulation                                                                    
      too = 0;   tff = 30;  % initial and final simulation time (min)
      ts = linspace(too,tff,31); % time vector for simulation
      rho = 62.4;           % water density (lbm/ft^3) 
      g = 1.1592e5;         % gravitational constant (ft/min^2)     
      wio = 1900;           % reference inlet flow rate (lbm/min)   
      ho = 10;              % reference height of fluid in tank (ft) 
      a = 5;                % surface area of tank (ft^2)
      ae = .02;             % area of exit pipe (ft^2) 
%   derived quantities
      c = rho*a;                      % capacitance
      c1nl = -rho*ae*sqrt(2*g)/c;     % fixed coeff in nonlinear eqn                                               
      r = sqrt(2*g*ho)/(rho*g*ae);    % resistance in linear eqn
      c1l = -1/(r*c);                 % fixed coeff in linear eqn
%   delta input                                                              
      dw1 = 0.1*wio;        % 10% change                                             
      dw2 = 0.5*wio;        % 50% change                                             
%
%   nonlinear simulation  -  impulse & step responses #1     
      y1o = ho+dw1/c;   ftx = @(t,x)  c1nl*sqrt(x)+wio/c;
      [t1i,y1i] = ode23(ftx,ts,y1o);                                 
      y1o = ho;   ftx = @(t,x)  c1nl*sqrt(x)+(wio+dw1)/c;
      [t1s,y1s] = ode23(ftx,ts,y1o);
%
%   linear simulation  -  impulse & step responses #1
      y1o = dw1/c;   ftx = @(t,x)  c1l*x;   
      [t2i,y2i] = ode23(ftx,ts,y1o);   y2i = ho+y2i;                                                           
      y1o = 0;   ftx = @(t,x)  c1l*x+dw1/c;   
      [t2s,y2s] = ode23(ftx,ts,y1o);   y2s = ho+y2s;                                                           
%
%   plot results for perturbation #1
      nfig = nfig+1;   figure(nfig)   
      subplot(2,2,1),plot(t1i,y1i,'r',t2i,y2i,'b--','LineWidth',2),grid on
      title('Impulse Response (10%)');
      xlabel('Time (min)'),ylabel('Height (ft)');                             
      legend('NL','L');                                     
      subplot(2,2,2),plot(t1s,y1s,'r',t2s,y2s,'b--','LineWidth',2),grid on
      title('Step Response (10%)');   
      xlabel('Time (min)'),ylabel('Height (ft)');                             
      legend('NL','L','Location','SouthEast');                                     
%
%   nonlinear simulation  -  impulse & step responses #2     
      y1o = ho+dw2/c;   ftx = @(t,x)  c1nl*sqrt(x)+wio/c;                                                           
      [t1i,y1i] = ode23(ftx,ts,y1o);                                 
      y1o = ho;   ftx = @(t,x)  c1nl*sqrt(x)+(wio+dw2)/c;                                              
      [t1s,y1s] = ode23(ftx,ts,y1o);                                 
%
%   linear simulation  -  impulse & step responses #2     
      y1o = dw2/c;   ftx = @(t,x)  c1l*x;                                                         
      [t2i,y2i] = ode23(ftx,ts,y1o);   y2i = ho+y2i;                                                           
      y1o = 0;   ftx = @(t,x)  c1l*x+dw2/c;                                                         
      [t2s,y2s] = ode23(ftx,ts,y1o);   y2s = ho+y2s;                                                           
%
%   plot results for perturbation #2                                                            
      subplot(2,2,3),plot(t1i,y1i,'r',t2i,y2i,'b--','LineWidth',2),grid on
      title('Impulse Response (50%)');
      xlabel('Time (min)'),ylabel('Height (ft)');                             
      legend('NL','L');                                     
      subplot(2,2,4),plot(t1s,y1s,'r',t2s,y2s,'b--','LineWidth',2),grid on
      title('Step Response (50%)');   
      xlabel('Time (min)'),ylabel('Height (ft)');                             
      legend('NL','L','Location','SouthEast');                                     
%
%   frequency domain plots for linearized model
%   Note: Matlab's bode command sets the frequency to rads/sec, but here we have
%   rads/min, so we need to do the plot manually (otherwise labels will be wrong).
      b = r;    a = [r*c 1];    sys1 = tf(b,a);
      w = logspace(-2,2,200);          % frequency domain (rad/min)
      nfig = nfig+1;  figure(nfig)
      [mag,phase] = bode(sys1,w);  
      mag1 = zeros(size(w));   phase1 = zeros(size(w));
      mag1(:) = 20*log10(mag(1,1,:));  % convert to db        
      phase1(:) = phase(1,1,:);        % also need to write as 2-D array for plots       
      subplot(2,1,1),semilogx(w,mag1,'b','LineWidth',2),grid  
      ylabel('Magnitude (dB)'),title('Bode Diagram')
      subplot(2,1,2),semilogx(w,phase1,'b','LineWidth',2),grid  
      ylabel('Phase (deg)'),xlabel('Frequency (rads/min)')
%
      nfig = nfig+1;  figure(nfig)
      subplot(2,1,1),nichols(sys1,w,'b'),grid off
      subplot(2,1,2),nyquist(sys1,w,'b'),grid on
      axis([-0.01 0.02 -0.01 0.01]);
      h = findobj(gcf,'color','b'); nh = length(h); 
      for i = 1:nh; set(h(i),'LineWidth',2); end
%   if the real and imaginary components are needed, then...
      [Re,Im] = nyquist(sys1,w);  Resys1(:,1) = Re(1,1,:);  Imsys1(:,1) = Im(1,1,:);
%
%   end of simulation
