%
%   TANKCS2.M  This file simulates the two tank system described in Section 6B 
%              of the Lecture Notes (see notes for detailed explanation)
%
%   Simulation of Simple Fluid Flow Systems (Case Study B - Case 2)
%
%   File prepared by J. R. White, UMass-Lowell (last update: Feb. 2020)
%

      clear all,   close all,   nfig = 0;
%
%   time and frequency vectors for simulation                                                              
      too = 0;   tff = 60;   t = too:0.25:tff;  t = t'; % time domain (min)                                              
      w = logspace(-3,2,200);                           % frequency domain (rad/min)
%                                       
%   Example 1 data             
      wio = 1500;     % reference inlet flow rate (lbm/min)   
      dw = 0.1*wio;   % 10% change in input flow rate (lbm/min)  
      r1 = .01;       % resistance at exit of tank 1 (ft-min/lbm)  
      r2 = .01;       % resistance at exit of tank 2 (ft-min/lbm)
      c1 = 250;       % capacitance of tank 1 (lbm/ft)  
      c2 = 250;       % capacitance of tank 2 (lbm/ft)  
      h1 = 30;        % reference height of fluid in tank 1 (ft) 
      h2 = 15;        % reference height of fluid in tank 2 (ft) 
%
%   calculate coeffs and setup state space representation for Ex 1                                                                     
      a11 = -1/(r1*c1);  a12 = -a11;  a21 = 1/(r1*c2);  a22 = -(1/r1 + 1/r2)/c2;                                          
      b1 = 1/c1;   b2 = 0;                                                           
      cc1 = 0;     cc2 = 1/r2;    d = 0;                                                            
      am1 = [a11 a12; a21 a22];    bv1 = [b1 b2]';   cv1 = [cc1 cc2]; 
      sys1 = ss(am1,dw*bv1,cv1,d);                                                  
%                                                                       
%   Example 2  (increase volume by factor of 4)                                                           
      c1 = 4*c1;         % capacitance of tank 1 (lbm/ft)  
%
%   recalculate coeffs and setup new state space representation for Ex 2                                                                     
      a11 = -1/(r1*c1);  a12 = -a11;  a21 = 1/(r1*c2);  a22 = -(1/r1 + 1/r2)/c2;                                          
      b1 = 1/c1;   b2 = 0;                                                           
      cc1 = 0;     cc2 = 1/r2;    d = 0;                                                            
      am2 = [a11 a12; a21 a22];    bv2 = [b1 b2]';   cv2 = [cc1 cc2];                                                   
      sys2 = ss(am2,dw*bv2,cv2,d);                                                  
%
%   find impulse and step responses for both Ex 1 and Ex 2                                                    
      [yi1,t,xi1] = impulse(sys1,t);     [ys1,t,xs1] = step(sys1,t);                                 
      [yi2,t,xi2] = impulse(sys2,t);     [ys2,t,xs2] = step(sys2,t);                                 
%
%   plot time domain responses                                                       
      nfig = nfig+1; figure(nfig)
      subplot(2,2,1),plot(t,yi1,'r','LineWidth',2),grid
      title('Impulse Response (V_1 = V_r)');  
      xlabel('Time (min)'),ylabel('Del Flow (lbm/min)');                   
      subplot(2,2,2),plot(t,ys1,'r','LineWidth',2),grid
      title('Step Response (V_1 = V_r)');     
      xlabel('Time (min)'),ylabel('Del Flow (lbm/min)');                   
      subplot(2,2,3),plot(t,yi2,'b','LineWidth',2),grid
      title('Impulse Response (V_1 = 4*V_r)');
      xlabel('Time (min)'),ylabel('Del Flow (lbm/min)');                   
      subplot(2,2,4),plot(t,ys2,'b','LineWidth',2),grid
      title('Step Response (V_1 = 4*V_r)');   
      xlabel('Time (min)'),ylabel('Del Flow (lbm/min)');                   
%
%   generate Bode plots for both examples  
      nfig = nfig+1; figure(nfig)
      [mag,phase] = bode(sys1,w);      % system 1 (ref volume in tank 1)
      mag1 = zeros(size(w));   phase1 = zeros(size(w));
      mag1(:) = 20*log10(mag(1,1,:));  % convert to db        
      phase1(:) = phase(1,1,:);        % also need to write as 2-D array for plots       
      [mag,phase] = bode(sys2,w);      % system 2 (V1 = 4*Vref)
      mag2 = zeros(size(w));   phase2 = zeros(size(w));
      mag2(:) = 20*log10(mag(1,1,:));  % convert to db        
      phase2(:) = phase(1,1,:);        % also need to write as 2-D array for plots       
      subplot(2,1,1),semilogx(w,mag1,'r',w,mag2,'b--','LineWidth',2),grid  
      ylabel('Magnitude (dB)'),title('Bode Diagram')
      subplot(2,1,2),semilogx(w,phase1,'r',w,phase2,'b--','LineWidth',2),grid  
      ylabel('Phase (deg)'),xlabel('Frequency (rads/min)')
      legend('V_1 = V_r','V_1 = 4*V_r')
% 
%   more time domain plots: h1 & h2 vs time                                      
      nfig = nfig+1;  figure(nfig)                                                               
      subplot(2,2,1),plot(t,xi1(:,1),'r-',t,xi1(:,2),'b--','LineWidth',2),grid,           
      title('Impulse Response (V_1 = V_r)');                                
      xlabel('Time (min)'),ylabel('Del H (ft)'),                        
      legend('h_1(t)','h_2(t)');                                    
      subplot(2,2,2),plot(t,xs1(:,1),'r-',t,xs1(:,2),'b--','LineWidth',2),grid,           
      title('Step Response (V_1 = V_r)');                                   
      xlabel('Time (min)'),ylabel('Del H (ft)'),                        
      legend('h_1(t)','h_2(t)','Location','SouthEast');                                    
      subplot(2,2,3),plot(t,xi2(:,1),'r-',t,xi2(:,2),'b--','LineWidth',2),grid,           
      title('Impulse Response (V_1 = 4*V_r)');                              
      xlabel('Time (min)'),ylabel('Del H (ft)'),                        
      legend('h_1(t)','h_2(t)');                                    
      subplot(2,2,4),plot(t,xs2(:,1),'r-',t,xs2(:,2),'b--','LineWidth',2),grid,           
      title('Step Response (V_1 = 4*V_r)');                                 
      xlabel('Time (min)'),ylabel('Del H (ft)'),                        
      legend('h_1(t)','h_2(t)','Location','SouthEast');                                    
%
%   end of example
