%
%   TANKCS3.M  This file simulates a single tank system with liquid level control 
%              as described in Section 6B of the Lecture Notes (see notes for 
%              detailed explanation)
%
%   Simulation of Simple Fluid Flow Systems (Case Study B - Case 3)
%
%   File prepared by J. R. White, UMass-Lowell (last update: Feb. 2020)
%

      clear all,  close all,   nfig = 0;
%
%   time and frequency vectors for simulation                                 
      too = 0;   tff = 30;   t = too:0.2:tff;   % time domain (min)   
      w = logspace(-2,2,200);                   % frequency domain (rad/min)
%
%   case specific data                             
       r = .01;       % flow resistance (ft-min/lbm)                        
       c = 300;       % tank capacitance (lbm/ft)                        
       rc = r*c;      % time constant (min)
%
%   open loop step response (100 lbm/min)
       num = r;   den = [rc 1];  sys = tf(num,den);  yol = 100*step(sys,t);
       nfig = nfig+1;   figure(nfig)
       plot(t,yol,'r-','LineWidth',2),grid; 
       title('Open Loop Response (100 lbm/min step change in inlet mass flow rate)');                  
       xlabel('Time (min)'),ylabel('Height (ft)');       
%                                                          
%   step response simulations (separate proportional and integral control)
%     kp has units of lbm/(ft-min)   and   ki has units of lbm/(ft-min^2)     
%       
%   case 1  k = kp = ki = 1         
       k = 1;   kr = k*r;   num1 = kr;                                 
       denp1 = [rc kr+1];   deni1 = [rc 1 kr]; 
       sysp1 = tf(num1,denp1);   sysi1 = tf(num1,deni1);    
       yp1 = step(sysp1,t);   yi1 = step(sysi1,t);         
%   case 2  k = kp = ki = 10  
       k = 10;   kr = k*r;   num2 = kr;                 
       denp2 = [rc kr+1];   deni2 = [rc 1 kr];                
       sysp2 = tf(num2,denp2);   sysi2 = tf(num2,deni2);   
       yp2 = step(sysp2,t);   yi2 = step(sysi2,t);       
%   case 3  k = kp = ki = 100       
       k = 100;   kr = k*r;   num3 = kr;        
       denp3 = [rc kr+1];   deni3 = [rc 1 kr];       
       sysp3 = tf(num3,denp3);   sysi3 = tf(num3,deni3);    
       yp3 = step(sysp3,t);   yi3 = step(sysi3,t);      
%   case 4  k = kp = ki = 1000              
       k = 1000;   kr = k*r;   num4 = kr;            
       denp4 = [rc kr+1];   deni4 = [rc 1 kr];     
       sysp4 = tf(num4,denp4);   sysi4 = tf(num4,deni4);  
       yp4 = step(sysp4,t);   yi4 = step(sysi4,t);         
%   plot results           
       nfig = nfig+1;   figure(nfig)
       subplot(211)
       plot(t,yp1,'r-',t,yp2,'g:',t,yp3,'b--',t,yp4,'c-.','LineWidth',2),grid; 
       title('Proportional Control');                  
       legend('Kp=1','Kp=10','Kp=100','Kp=1000','Location','BestOutside');       
       ylabel('Height (ft)');       
       subplot(212)
       plot(t,yi1,'r-',t,yi2,'g:',t,yi3,'b--',t,yi4,'c-.','LineWidth',2),grid;   
       title('Integral Control');                   
       legend('Ki=1','Ki=10','Ki=100','Ki=1000','Location','BestOutside');   
       xlabel('Time (min)'),ylabel('Height (ft)');    
%
%   frequency domain Bode simulations (separate proportional & integral control)
%
%   Bode plots for proportional control 
       nfig = nfig+1;   figure(nfig)
       [mag,phase] = bode(sysp1,w);     % system 1
       mag1 = zeros(size(w));   phase1 = zeros(size(w));
       mag1(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase1(:) = phase(1,1,:);        % also need to write as 2-D array for plots       
       [mag,phase] = bode(sysp2,w);     % system 2 
       mag2 = zeros(size(w));   phase2 = zeros(size(w));
       mag2(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase2(:) = phase(1,1,:);        % also need to write as 2-D array for plots 
       [mag,phase] = bode(sysp3,w);     % system 3
       mag3 = zeros(size(w));   phase3 = zeros(size(w));
       mag3(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase3(:) = phase(1,1,:);        % also need to write as 2-D array for plots       
       [mag,phase] = bode(sysp4,w);     % system 4 
       mag4 = zeros(size(w));   phase4 = zeros(size(w));
       mag4(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase4(:) = phase(1,1,:);        % also need to write as 2-D array for plots 
%      
       subplot(2,1,1),semilogx(w,mag1,'r-',w,mag2,'g:', ...
                               w,mag3,'b--',w,mag4,'c-.','LineWidth',2),grid  
       ylabel('Magnitude (dB)'),title('Bode Diagram')
       subplot(2,1,2),semilogx(w,phase1,'r-',w,phase2,'g:', ...
                               w,phase3,'b--',w,phase4,'c-.','LineWidth',2),grid  
       ylabel('Phase (deg)'),xlabel('Frequency (rads/min)')
       legend('Kp=1','Kp=10','Kp=100','Kp=1000','Location','NorthEast');       
%
%   Bode plots for integral control 
       nfig = nfig+1;   figure(nfig)
       [mag,phase] = bode(sysi1,w);     % system 1
       mag1 = zeros(size(w));   phase1 = zeros(size(w));
       mag1(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase1(:) = phase(1,1,:);        % also need to write as 2-D array for plots       
       [mag,phase] = bode(sysi2,w);     % system 2 
       mag2 = zeros(size(w));   phase2 = zeros(size(w));
       mag2(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase2(:) = phase(1,1,:);        % also need to write as 2-D array for plots 
       [mag,phase] = bode(sysi3,w);     % system 3
       mag3 = zeros(size(w));   phase3 = zeros(size(w));
       mag3(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase3(:) = phase(1,1,:);        % also need to write as 2-D array for plots       
       [mag,phase] = bode(sysi4,w);     % system 4 
       mag4 = zeros(size(w));   phase4 = zeros(size(w));
       mag4(:) = 20*log10(mag(1,1,:));  % convert to db        
       phase4(:) = phase(1,1,:);        % also need to write as 2-D array for plots 
%      
       subplot(2,1,1),semilogx(w,mag1,'r-',w,mag2,'g:', ...
                               w,mag3,'b--',w,mag4,'c-.','LineWidth',2),grid  
       ylabel('Magnitude (dB)'),title('Bode Diagram')
       subplot(2,1,2),semilogx(w,phase1,'r-',w,phase2,'g:', ...
                               w,phase3,'b--',w,phase4,'c-.','LineWidth',2),grid  
       ylabel('Phase (deg)'),xlabel('Frequency (rads/min)')
       legend('Ki=1','Ki=10','Ki=100','Ki=1000','Location','NorthEast');       
%                                                          
%   step response simulations (combined PI control)
%     kp has units of lbm/(ft-min)   and   ki has units of lbm/(ft-min^2)     
%       
%   case 1  kp = ki = 100         
       kp = 100;   ki = 100;   
       numpi1 = [r*kp r*ki];   denpi1 = [rc (1+kp*r) r*ki];  roots(denpi1)                              
       syspi1 = tf(numpi1,denpi1);   ypi1 = step(syspi1,t);           
%       
%   case 2  kp = 100 and k2 = 100         
       kp = 250;   ki = 100;   
       numpi2 = [r*kp r*ki];   denpi2 = [rc (1+kp*r) r*ki];  roots(denpi2)                              
       syspi2 = tf(numpi2,denpi2);   ypi2 = step(syspi2,t);           
%      
%   plot results           
       nfig = nfig+1;   figure(nfig)
       plot(t,yp3,'r-',t,yi3,'g:',t,ypi1,'b--',t,ypi2,'c-.','LineWidth',2),grid; 
       title('Comparison of P, I, and PI Control');                  
       ylabel('Height (ft)');       
       xlabel('Time (min)'),ylabel('Height (ft)'); 
       legend('Kp = 100','Ki = 100','Kp= 100, Ki=100','Kp= 250, Ki=100');       
%
%   end of example